from enum import Enum
from typing import List, Optional

from cognite.airworkflow.util import env
from cognite.airworkflow.util.projecthelpers import project_name_finder


class Project(Enum):
    NAME = project_name_finder()

    @classmethod
    def _missing_(cls, value):
        raise ValueError(f"{value} is not a valid AIR project, must be one of {[p.value for p in Project]}")


class ProjectInfo:
    def __init__(
        self,
        project: str,
        client_key_name: Optional[str] = None,
        base_url: Optional[str] = None,
        tenant_id: Optional[str] = None,
        client_id: Optional[str] = None,
        secret_name: Optional[str] = None,
        token_url: Optional[str] = None,
        scopes: Optional[List[str]] = None,
    ):
        self.name: str = project
        self.project: str = project
        self._base_url: Optional[str] = base_url
        # legacy
        self.client_key_name: Optional[str] = client_key_name
        # oidc
        self.tenant_id: Optional[str] = tenant_id
        self.client_id: Optional[str] = client_id
        self.secret_name: Optional[str] = secret_name
        self.token_url: Optional[str] = token_url
        self.scopes: Optional[List[str]] = scopes
        # validate
        self.validate_auth()

    @property
    def oidc(self):
        return all(
            self.tenant_id and self.client_id and self.secret_name and self.token_url and self.scopes
        ) and isinstance(self.scopes, list)

    def validate_auth(self):
        if not self.client_key_name and not self.oidc:
            raise ValueError("Either legacy or OIDC authentication needs to be specified.")

    @property
    def base_url(self):
        return self._base_url or "https://api.cognitedata.com"

    def get_client_api_key(self) -> str:
        return env.get_env_value(self.client_key_name)

    def get_client_secret(self) -> str:
        return env.get_env_value(self.secret_name)



import subprocess

import yaml
from constants import REPO_CONFIG_PROJECT_FUNCTION_MAP

from cognite.airworkflow.constants import CHANGED_CODE_SCRIPT_PATH, REPO_CONFIG
from cognite.airworkflow.master_config import get_master_config
from cognite.airworkflow.util import cdf, env


def file_path_changed(pathvalue: str):
    call_bash = subprocess.Popen(["bash", CHANGED_CODE_SCRIPT_PATH, pathvalue], stdout=subprocess.PIPE)
    output = call_bash.communicate()[0]
    encoding = "utf-8"
    return output.decode(encoding)


def check_deployment(function_name: str, project_name: str):
    client = cdf.experimental_client(project_name)
    model_asset = client.assets.retrieve(external_id=function_name)
    if model_asset is None:
        return "True"

    # check if function has been deployed
    repo_name = env.get_env_value("GITHUB_REPOSITORY")
    function_external_id = str(repo_name) + "/" + str(function_name) + ":latest"

    function_asset = client.functions.retrieve(external_id=function_external_id)
    if function_asset is None:
        return "True"

    cdf_model_version = model_asset.metadata["modelVersion"]
    configuration = get_master_config()[1][project_name]
    key1 = [i for i in configuration.keys() if i.parts[-1] == function_name][0]
    model_version = configuration[key1].model_version
    model_version = ".".join(model_version.split(".")[:-1])
    already_deployed = str(model_version != cdf_model_version)
    return already_deployed


def final_string_generator():
    # Read files and send to function
    # CR
    filtered_list = []

    with open(REPO_CONFIG, "r") as f:
        valuesYaml = yaml.load(f, Loader=yaml.FullLoader)

    function_project_mapping = []
    for i in valuesYaml[REPO_CONFIG_PROJECT_FUNCTION_MAP]:
        tenant_name_list = valuesYaml[REPO_CONFIG_PROJECT_FUNCTION_MAP][i]
        function_name_list = [i] * len(tenant_name_list)
        result_string = list(map(lambda x, y: str(x) + "@" + str(y), function_name_list, tenant_name_list))
        function_project_mapping.extend(result_string)

    for i in function_project_mapping:
        function_name, project_name = i.split("@")
        deployment_boolean = check_deployment(function_name, project_name)
        file_path_changed_boolean = file_path_changed("functions/" + function_name)
        file_path_changed_boolean = file_path_changed_boolean.rstrip()
        if file_path_changed_boolean == "True" or deployment_boolean == "True":
            filtered_list.append('"' + i + '"')
    if not filtered_list:
        print(str(True))
        return str(True)
    else:
        print(str(False))
        return str(False)


if __name__ == "__main__":
    final_string_generator()



from pathlib import Path
from typing import Dict

from cognite.airworkflow.model.project import Project, ProjectInfo
from cognite.airworkflow.util import env, repoconfighelpers

# List of base constants
working_path = env.get_env_value("PWD")
ROOT_DIR = Path(working_path)
FUNCTIONS_PATH = ROOT_DIR / "functions"
IGNORE_MODELS_PATH = ROOT_DIR / ".ignore_models"
SCHEDULE_MANAGER_EXTERNAL_IDS = [
    "cognitedata/air-ds-infrastructure/schedulemanager:latest",
    "cognitedata/air-functions/schedulemanager:latest",
    "cognitedata/air-ds-infrastructure/schedulemanager2:latest",
]

REPO_CONFIG = ROOT_DIR / "repoconfig.yaml"

REPO_CONFIG_PROJECT_FUNCTION_MAP = "ProjectFunctionMap"
# Paths within a functions folder
FUNCTION_REL_PATH = Path("function")
FUNCTION_REL_CONFIG_PATH = FUNCTION_REL_PATH / "config.yaml"
FUNCTION_REL_RESOURCE_PATH = FUNCTION_REL_PATH / "resources"
FUNCTION_REL_DEPLOYMENT_PATH = FUNCTION_REL_RESOURCE_PATH / "dependencies.yaml"
FUNCTION_REL_INIT_PATH = FUNCTION_REL_PATH / "__init__.py"

# WORK FLOW PATHS
WORK_FLOW_PATH = ROOT_DIR / ".github" / "workflows"
WORK_FLOWS = [WORK_FLOW_PATH / p for p in ["build-master.yaml", "build-pr.yaml", "delete-pr.yaml"]]

# CONFIG PATHS
BASE_PATH = Path(__file__).parent.resolve()
CHANGED_CODE_SCRIPT_PATH = BASE_PATH / "file_changed.sh"
SCHEMAS_PATH = BASE_PATH / "schemas"
DEPLOYMENT_SCHEMA_PATH = SCHEMAS_PATH / "deployment-schema.yaml"
CONFIG_SCHEMA_PATH = SCHEMAS_PATH / "config-schema.yaml"

url_dict = repoconfighelpers.get_url_dict()
AUTHENTICATION: Dict = {}
for project in Project.NAME.value:
    if project in url_dict:
        project_config = url_dict[project]
        if isinstance(project_config, list):  # legacy
            project_authentication = ProjectInfo(
                project=project, client_key_name=project_config[1], base_url=str(project_config[0])
            )
        elif isinstance(project_config, dict):  # OIDC
            project_authentication = ProjectInfo(
                project=project,
                base_url=project_config["base_url"],
                tenant_id=project_config["tenant_id"],
                client_id=project_config["client_id"],
                secret_name=project_config["secret_name"],
                token_url=project_config["secret_name"],
                scopes=project_config["scopes"],
            )
        else:
            project_authentication = ProjectInfo(project, "AIR_API_KEY", str(project_config))

    else:
        project_authentication = ProjectInfo(project, "AIR_API_KEY")
    AUTHENTICATION[project] = project_authentication


# GitHub/Lab constants
gitlab_function_name_addition = "FUNCTION_NAME_ADDITION"
gitlab_ci_project = "CI_PROJECT_PATH"

# String variable definitions
COGNITE_CLIENT_NAME = "AIR Functions Client"
